// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Runtime.Serialization;

using LoCoL.Exceptions;

namespace LoCoL.Collections
{
	/// <summary>
	///    .
	/// </summary>
	[Serializable]
	public class AutomatonActionArray : IEnumerable, IChangeable, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public AutomatonActionArray()
		{
			m_Array = new ArrayList();
		}

		/// <summary>
		/// .
		/// </summary>
		public AutomatonActionArray( params IAutomatonAction[] actions )
		{
			m_Array = new ArrayList();
			foreach ( IAutomatonAction a in actions )
			{
				this.Add( a );
			}
		}

		/// <summary>
		///  .
		/// </summary>
		/// <param name="actions">,    .</param>
		public AutomatonActionArray( AutomatonActionArray actions )
		{
			m_Array = (ArrayList)actions.m_Array.Clone();
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected AutomatonActionArray(
			SerializationInfo info,
			StreamingContext context )
		{
			m_Array = new ArrayList();
			for ( int i = 0; i < info.MemberCount; i++ )
			{
				IAutomatonAction a = (IAutomatonAction)info.GetValue(
					"Value_" + i,
					typeof(IAutomatonAction) );
				this.Add( a );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			int i = 0;
			foreach ( IAutomatonAction a in m_Array )
			{
				info.AddValue( "Value_" + i, a );
				i++;
			}
		}

		#endregion

		#region Private Members

		/// <summary>
		///  ,    .
		/// </summary>
		private ArrayList m_Array;

		#endregion

		#region Public Methods

		/// <summary>
		///  .
		/// </summary>
		public AutomatonActionArray Clone()
		{
			return new AutomatonActionArray( this );
		}

		/// <summary>
		///        
		/// <c>automaton</c>.
		/// </summary>
		/// <param name="automaton">
		/// ,    .
		/// </param>
		public void DoActions( Automaton automaton )
		{
			foreach ( IAutomatonAction action in this )
			{
				action.DoAction( automaton );
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public void Clear()
		{
			m_Array.Clear();
		}

		/// <summary>
		///  .
		/// </summary>
		/// <param name="action"> .</param>
		public int Add( IAutomatonAction action )
		{
			int index = m_Array.Add( action );
			RaiseChangedEvent( this, ChangedEventArgs.Empty );
			return index;
		}

		/// <summary>
		/// ,     .
		/// </summary>
		/// <param name="action"> .</param>
		public bool Contains( IAutomatonAction action )
		{
			return m_Array.Contains( action );
		}

		/// <summary>
		///    .
		///   ,  .
		/// </summary>
		/// <param name="action"> .</param>
		/// <exception cref="LoCoLCollectionsException">
		///    .
		/// </exception>
		public void Remove( IAutomatonAction action )
		{
			if ( Contains( action ) )
			{
				m_Array.Remove( action );
				RaiseChangedEvent( this, ChangedEventArgs.Empty );
			}
			else
			{
				throw new LoCoLCollectionsException(
					"   ." );
			}
		}

		#endregion

		#region Public Properties

		/// <summary>
		///  .
		/// </summary>
		public IAutomatonAction this[ int index ]
		{
			get
			{
				return (IAutomatonAction)m_Array[ index ];
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public int Count
		{
			get
			{
				return m_Array.Count;
			}
		}

		#endregion

		#region IEnumerable Implementation

		/// <summary>
		/// Enumerator.
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return m_Array.GetEnumerator();
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,      .
		/// </summary>
		public event ChangedEventHandler Changed;

		/// <summary>
		///   <see cref="Changed"/>.
		/// </summary>
		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}

