// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Threading;
using System.IO;
using System.Runtime.Serialization;

using LoCoL.Collections;
using LoCoL.Exceptions;
using LoCoL.Logs;

namespace LoCoL
{

	/// <summary>
	///  .
	/// </summary>
	[Serializable]
	public class AutomatonManager : IValidCheckable, IChangeable, ISerializable
	{

		#region Public Constructor

		/// <summary>
		/// .
		/// </summary>
		public AutomatonManager()
		{
			m_Automatons = new AutomatonSet();
			m_Automatons.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_Automatons.Changed +=
				new ChangedEventHandler( OnAutomatonsChanged );
			m_Notifications = new NotificationSet();
			m_Notifications.Add( new Notification( 0, "<Reserved>" ) );
			m_Notifications.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_Timers = new Hashtable();
			m_MessageQueues = Hashtable.Synchronized( new Hashtable() );
			m_IsCycle = false;
			m_CurrentCycle = -1;
			m_RunnedAutomatons = new AutomatonSet();
			m_AutomatonStates = new Hashtable();
			m_Checkers = new ArrayList();
			m_IsValid = false;
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected AutomatonManager(
			SerializationInfo info,
			StreamingContext context )
		{
			m_Automatons = (AutomatonSet)info.GetValue(
				"Automatons",
				typeof(AutomatonSet) );
			m_Automatons.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_Automatons.Changed +=
				new ChangedEventHandler( OnAutomatonsChanged );
			m_Notifications = (NotificationSet)info.GetValue(
				"Notifications",
				typeof(NotificationSet) );
			m_Notifications.Changed +=
				new ChangedEventHandler( RaiseChangedEvent );
			m_Timers = new Hashtable();
			m_MessageQueues = Hashtable.Synchronized( new Hashtable() );
			m_IsCycle = false;
			m_CurrentCycle = -1;
			m_RunnedAutomatons = new AutomatonSet();
			m_AutomatonStates = new Hashtable();
			m_Checkers = new ArrayList();
			m_IsValid = false;
			m_Title = info.GetString( "Title" );
			m_Description = info.GetString( "Description" );
			m_Copyright = info.GetString( "Copyright" );
			m_Version = (Version)info.GetValue( "Version", typeof(Version) );
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			info.AddValue( "Title", m_Title );
			info.AddValue( "Description", m_Description );
			info.AddValue( "Copyright", m_Copyright );
			info.AddValue( "Version", m_Version );
			info.AddValue( "Notifications", m_Notifications );
			info.AddValue( "Automatons", m_Automatons );
		}

		#endregion

		#region Private Members

		/// <summary>
		///  .    .
		/// </summary>
		private bool m_IsValid;

		/// <summary>
		///         ,
		///   .
		/// </summary>
		private Hashtable m_AutomatonStates;

		/// <summary>
		///   ,   .
		/// </summary>
		private bool m_IsCycle;
		
		/// <summary>
		///      .
		/// </summary>
		private int m_CurrentCycle;

		/// <summary>
		///     .
		/// </summary>
		private Hashtable m_MessageQueues;

		/// <summary>
		///     .
		/// </summary>
		private AutomatonSet m_RunnedAutomatons;

		/// <summary>
		///   .
		/// </summary>
		private Notification m_CurrentNotification;

		/// <summary>
		///     .
		/// </summary>
		private Hashtable m_Timers;

		/// <summary>
		/// ID  .
		/// </summary>
		private int m_NextTimerID;

		/// <summary>
		///  ,     .
		/// </summary>
		private ArrayList m_Checkers;

		/// <summary>
		///  ,  ,
		///   .
		/// </summary>
		private AutomatonSet m_Automatons;

		/// <summary>
		///    .
		/// </summary>
		private NotificationSet m_Notifications;

		/// <summary>
		///   .
		/// </summary>
		private string m_Title;

		/// <summary>
		///   .
		/// </summary>
		private string m_Description;

		/// <summary>
		///   .
		/// </summary>
		private string m_Copyright;

		/// <summary>
		///   .
		/// </summary>
		private Version m_Version;

		#endregion

		#region Private Class

		/// <summary>
		///  .     .
		/// </summary>
		private class TimerCallbackState
		{
			/// <summary>
			/// .
			/// </summary>
			/// <param name="automatsID"> ID .</param>
			/// <param name="typeOfNotification"> .</param>
			public TimerCallbackState(
				int[] automatsID,
				NotificationType typeOfNotification )
			{
				if ( automatsID != null )
				{
					AutomatsID = (int[])automatsID.Clone();
				}
				TypeOfNotification = typeOfNotification;
			}
			/// <summary>
			///  ID .
			/// </summary>
			public int[] AutomatsID;
			/// <summary>
			///  .
			/// </summary>
			public NotificationType TypeOfNotification;
		}

		#endregion

		#region Private Methods

		/// <summary>
		///       .
		/// </summary>
		/// <param name="automatonID">ID .</param>
		/// <returns>    .</returns>
		/// <exception cref="AutomatonManagerException">
		///    ID   .
		/// </exception>
		private Queue MessageQueue( int automatonID )
		{
			if ( Automatons[ automatonID ].Equals( null ) )
			{
				throw new AutomatonManagerException(
					"   ID   ." );
			}
			Queue q;
			if ( m_MessageQueues[ automatonID ] == null )
			{
				m_MessageQueues[ automatonID ] =
					q =
					Queue.Synchronized( new Queue() );
			}
			else
			{
				q = (Queue)m_MessageQueues[ automatonID ];
			}
			return q;
		}

		/// <summary>
		///     .
		/// </summary>
		/// <param name="automaton"> .</param>
		/// <param name="notification">
		///   .
		/// </param>
		/// <remarks>
		///    
		///     (thread-safe).
		/// </remarks>
		/// <exception cref="AutomatonManagerException">
		///    :
		///        .
		/// </exception>
		private void Run( Automaton automaton, Notification notification )
		{
			lock ( this )
			{
				//   .
				CheckValid();
				// ,     .
				bool isAutoBeginCycle = false;
				//    .
				if ( !IsCycle )
				{
					//     ,
					//    .
					BeginCycle();
					isAutoBeginCycle = true;
				}
				//  .
				if ( m_RunnedAutomatons.Contains( automaton ) )
				{
					throw new AutomatonManagerException(
						"   : " +
						" " + automaton.ToString() +
						"     ." );
				}
				//         .
				m_RunnedAutomatons.Add( automaton );
				//    .
				m_CurrentNotification = notification;
				//    .
				LogBeginNotificationProcessing( automaton, notification );
				//    .
				Transition tr = automaton.SelectTransition();
				//   .
				if ( tr != null )
				{
					//  .
					automaton.DoTransition( tr );
				}
				//    .
				LogEndNotificationProcessing( automaton, notification );
				//    .
				m_CurrentNotification = null;
				//    .
				if ( isAutoBeginCycle )
				{
					//    .
					EndCycle();
				}
			}
		}

		/// <summary>
		///    .
		///      (     ),
		///   ,     .
		/// </summary>
		private void OnAutomatonsChanged( object sender, ChangedEventArgs e )
		{
			if ( ReferenceEquals( sender, m_Automatons ) )
			{
				foreach ( Automaton a in m_Automatons )
				{
					a.SetManager( this );
				}
			}
		}

		/// <summary>
		///  .
		/// </summary>
		/// <param name="state">  TimerCallbackState.</param>
		private void TimerCallback( object state )
		{
			TimerCallbackState st = (TimerCallbackState)state;
			foreach ( int id in st.AutomatsID )
			{
				if ( st.TypeOfNotification == NotificationType.NotSpecified )
				{
					Run( id );
				}
				else
				{
					Run( Automatons[ id ],
						Notifications[ 0 ].CreateNotification(
							st.TypeOfNotification ) );
				}
			}
		}

		#endregion

		#region Internal Methods

		/// <summary>
		/// ,       .
		/// </summary>
		/// <param name="notificationID">ID .</param>
		/// <param name="notificationType"> .</param>
		/// <returns> .</returns>
		/// <exception cref="AutomatonManagerException">
		///         .
		/// </exception>
		internal bool CheckNotification(
			int notificationID,
			NotificationType notificationType )
		{
			if ( m_CurrentNotification == null )
			{
				throw new AutomatonManagerException( "   " +
					"     ." );
			}
			if ( m_CurrentNotification.ID != notificationID )
			{
				return false;
			}
			if ( notificationType == NotificationType.NotSpecified ||
				m_CurrentNotification.Type == notificationType )
			{
				return true;
			}
			return false;
		}

		/// <summary>
		/// ,       .
		///    ,  ,
		///        .
		/// </summary>
		/// <param name="automatonID">ID .</param>
		/// <param name="stateID">ID .</param>
		/// <returns> .</returns>
		internal bool CheckState( int automatonID, int stateID )
		{
			int st;
			if ( m_AutomatonStates[ automatonID ] != null )
			{
				st = (int)m_AutomatonStates[ automatonID ];
			}
			else
			{
				st = m_Automatons[ automatonID ].CurrentState.ID;
			}
			return st == stateID;
		}

		#endregion

		#region Public Methods

		/// <summary>
		///   .
		/// </summary>
		/// <remarks>
		///    .
		/// </remarks>
		/// <exception cref="AutomatonManagerException">
		///        .
		/// </exception>
		public void Reset()
		{
			if ( m_IsCycle )
			{
				throw new AutomatonManagerException( "  " +
					"     ." );
			}
			foreach ( Automaton a in Automatons )
			{
				a.Reset();
			}
		}

		/// <summary>
		///   .
		/// </summary>
		/// <exception cref="AutomatonManagerException">
		///   .
		/// </exception>
		public void BeginCycle()
		{
			if ( m_IsCycle )
			{
				throw new AutomatonManagerException( "  ." );
			}
			m_IsCycle = true;
			m_CurrentCycle++;
			m_RunnedAutomatons.Clear();
			m_AutomatonStates.Clear();
			LogBeginCycle();
		}

		/// <summary>
		///   .
		/// </summary>
		/// <exception cref="AutomatonManagerException">
		///    .
		/// </exception>
		public void EndCycle()
		{
			if ( !m_IsCycle )
			{
				throw new AutomatonManagerException( "   ." );
			}
			m_IsCycle = false;
			LogEndCycle();
		}

		/// <summary>
		///   .
		///    .
		/// </summary>
		/// <param name="automatonID">ID .</param>
		/// <param name="notificationID">ID .</param>
		/// <exception cref="AutomatonManagerException">
		///    ID    .
		/// </exception>
		public void SendMessage( int automatonID, int notificationID )
		{
			if ( Notifications[ notificationID ] == null )
			{
				throw new AutomatonManagerException( "   ID " +
					"   ." );
							  }
			MessageQueue( automatonID ).Enqueue( notificationID );
			LogSendMessage(
				Automatons[ automatonID ],
				Notifications[ notificationID ].CreateMessage() );
		}

		/// <summary>
		///   .
		///     .
		/// </summary>
		/// <param name="automatonID">ID .</param>
		/// <param name="notificationID">ID .</param>
		/// <exception cref="AutomatonManagerException">
		///    ID    .
		/// </exception>
		public void RaiseEvent( int automatonID, int notificationID )
		{
			if ( Notifications[ notificationID ] == null )
			{
				throw new AutomatonManagerException( "   ID " +
					"   ." );
			}
			Run( Automatons[ automatonID ],
				Notifications[ notificationID ].CreateEvent() );
		}

		/// <summary>
		///       ,
		///   ,    m0.
		/// </summary>
		/// <param name="automatonID">ID .</param>
		public void Run( int automatonID )
		{
			int id =
				( MessageQueue( automatonID ).Count == 0 ) ?
				( 0 ) :
				( (int)MessageQueue( automatonID ).Dequeue() );
			Run( Automatons[ automatonID ],
				Notifications[ id ].CreateMessage() );
		}

		/// <summary>
		///  ,     .
		/// </summary>
		/// <exception cref="AutomatonException">
		///  ,  ,  .
		/// </exception>
		public void RegisterChecker( IAutomatonChecker checker )
		{
			if ( m_Checkers.Contains( checker ) )
			{
				throw new AutomatonException( " , " +
					" ,  ." );
			}
			m_Checkers.Add( checker );
		}

		/// <summary>
		///     ,
		///     .
		/// </summary>
		/// <exception cref="AutomatonException">
		///  ,  ,   .
		/// </exception>
		public void UnregisterChecker( IAutomatonChecker checker )
		{
			if ( !m_Checkers.Contains( checker ) )
			{
				throw new AutomatonException( " , " + 
					" ,   ." );
			}
			m_Checkers.Remove( checker );
		}

		/// <summary>
		///      
		///  <c>m0</c>   .
		/// </summary>
		/// <param name="automatsID"> ID .</param>
		/// <param name="millisec">
		///  ,  
		///    .
		/// </param>
		/// <returns>ID .</returns>
		public int SetMessageTimer( int[] automatsID, int millisec )
		{
			Timer timer = new Timer(
				new TimerCallback( TimerCallback ),
				new TimerCallbackState( automatsID, NotificationType.Message ),
				millisec,
				millisec );
			m_NextTimerID++;
			m_Timers[ m_NextTimerID ] = timer;
			return m_NextTimerID;
		}

		/// <summary>
		///        
		///   ( <c>m0</c>,   ).
		/// </summary>
		/// <param name="automatsID"> ID .</param>
		/// <param name="millisec">
		///  ,    
		///  .
		/// </param>
		/// <returns>ID .</returns>
		public int SetRunTimer( int[] automatsID, int millisec )
		{
			Timer timer = new Timer(
				new TimerCallback( TimerCallback ),
				new TimerCallbackState(
					automatsID,
					NotificationType.NotSpecified ),
				millisec,
				millisec );
			m_NextTimerID++;
			m_Timers[ m_NextTimerID ] = timer;
			return m_NextTimerID;
		}

		/// <summary>
		///      
		///   <c>e0</c>   .
		/// </summary>
		/// <param name="automatsID"> ID .</param>
		/// <param name="millisec">
		///  ,    
		///  .
		/// </param>
		/// <returns>ID .</returns>
		public int SetEventTimer( int[] automatsID, int millisec )
		{
			Timer timer = new Timer(
				new TimerCallback( TimerCallback ),
				new TimerCallbackState( automatsID, NotificationType.Event ),
				millisec,
				millisec );
			m_NextTimerID++;
			m_Timers[ m_NextTimerID ] = timer;
			return m_NextTimerID;
		}

		/// <summary>
		///  .
		/// </summary>
		/// <param name="timerID">ID .</param>
		/// <exception cref="AutomatonManagerException">
		///    ID  .
		/// </exception>
		public void KillTimer( int timerID )
		{
			if ( !m_Timers.ContainsKey( timerID ) )
			{
				throw new AutomatonManagerException(
					"   ID  ." );
			}
			Timer timer = (Timer)m_Timers[ timerID ];
			m_Timers.Remove( timerID );
			timer.Dispose();
		}

		/// <summary>
		///   <c>logger</c>   .
		/// </summary>
		/// <param name="logger"> .</param>
		public void RegisterLogger( ILogger logger )
		{
			BeginCycleEvent +=
				new CycleEventHandler(
					logger.OnBeginCycle );
			BeginNotificationProcessingEvent +=
				new NotificationProcessingEventHandler(
					logger.OnBeginNotificationProcessing );
			InputInquireEvent +=
				new InputInquireEventHandler(
					logger.OnInputInquire );
			NotificationCheckingEvent +=
				new NotificationCheckingEventHandler(
					logger.OnNotificationChecking );
			StateCheckingEvent +=
				new StateCheckingEventHandler(
					logger.OnStateChecking );
			TransitionEvent +=
				new TransitionEventHandler(
					logger.OnTransition );
			OutputInvocationEvent +=
				new OutputInvocationEventHandler(
					logger.OnOutputInvocation );
			SendMessageEvent +=
				new SendMessageEventHandler(
					logger.OnSendMessage );
			EndNotificationProcessingEvent +=
				new NotificationProcessingEventHandler(
					logger.OnEndNotificationProcessing );
			EndCycleEvent +=
				new CycleEventHandler(
					logger.OnEndCycle );
		}

		/// <summary>
		///   <c>logger</c>   .
		/// </summary>
		/// <param name="logger"> .</param>
		public void UnregisterLogger( ILogger logger )
		{
			BeginCycleEvent -=
				new CycleEventHandler(
					logger.OnBeginCycle );
			BeginNotificationProcessingEvent -=
				new NotificationProcessingEventHandler(
					logger.OnBeginNotificationProcessing );
			InputInquireEvent -=
				new InputInquireEventHandler(
					logger.OnInputInquire );
			NotificationCheckingEvent -=
				new NotificationCheckingEventHandler(
					logger.OnNotificationChecking );
			StateCheckingEvent -=
				new StateCheckingEventHandler(
					logger.OnStateChecking );
			TransitionEvent -=
				new TransitionEventHandler(
					logger.OnTransition );
			OutputInvocationEvent -=
				new OutputInvocationEventHandler(
					logger.OnOutputInvocation );
			SendMessageEvent -=
				new SendMessageEventHandler(
					logger.OnSendMessage );
			EndNotificationProcessingEvent -=
				new NotificationProcessingEventHandler(
					logger.OnEndNotificationProcessing );
			EndCycleEvent -=
				new CycleEventHandler(
					logger.OnEndCycle );
		}

		#endregion

		#region Public Properties

		/// <summary>
		///   ,   .
		/// </summary>
		public bool IsCycle
		{
			get
			{
				return m_IsCycle;
			}
		}

		/// <summary>
		///      .
		/// </summary>
		public int CurrentCycle
		{
			get
			{
				return m_CurrentCycle;
			}
		}

		/// <summary>
		///  ,  ,
		///   .
		/// </summary>
		public AutomatonSet Automatons
		{
			get
			{
				return m_Automatons;
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public NotificationSet Notifications
		{
			get
			{
				return m_Notifications;
			}
		}


		/// <summary>
		///   .
		/// </summary>
		public string Title
		{
			get
			{
				return m_Title;
			}
			set
			{
				m_Title = value;
			}
		}

		/// <summary>
		///   .
		/// </summary>
		public string Description
		{
			get
			{
				return m_Description;
			}
			set
			{
				m_Description = value;
			}
		}

		/// <summary>
		///   .
		/// </summary>
		public string Copyright
		{
			get
			{
				return m_Copyright;
			}
			set
			{
				m_Copyright = value;
			}
		}

		/// <summary>
		///   .
		/// </summary>
		public Version Version
		{
			get
			{
				return m_Version;
			}
			set
			{
				m_Version = value;
			}
		}

		#endregion

		#region Private and Internal Log Methods

		/// <summary>
		///    .
		/// </summary>
		private void LogBeginCycle()
		{
			if ( BeginCycleEvent != null )
			{
				BeginCycleEvent( this );
			}
		}

		/// <summary>
		///      .
		/// </summary>
		internal void LogBeginNotificationProcessing(
			Automaton automaton,
			Notification notification )
		{
			if ( BeginNotificationProcessingEvent != null )
			{
				BeginNotificationProcessingEvent( automaton, notification );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		internal void LogInputInquire(
			Automaton automaton,
			InputVariable variable,
			bool result )
		{
			if ( InputInquireEvent != null )
			{
				InputInquireEvent( automaton, variable, result );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		internal void LogNotificationChecking(
			Automaton automaton,
			Notification notification,
			bool result )
		{
			if ( NotificationCheckingEvent != null )
			{
				NotificationCheckingEvent( automaton, notification, result );
			}
		}

		/// <summary>
		///       .
		/// </summary>
		internal void LogStateChecking(
			Automaton automaton,
			Automaton checkedAutomaton,
			State state,
			bool result )
		{
			if ( StateCheckingEvent != null )
			{
				StateCheckingEvent(
					automaton,
					checkedAutomaton,
					state,
					result );
			}
		}

		/// <summary>
		///   .
		/// </summary>
		internal void LogTransition(
			Automaton automaton,
			Transition transition )
		{
			if ( TransitionEvent != null )
			{
				TransitionEvent( automaton, transition );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		internal void LogOutputInvocation(
			Automaton automaton,
			OutputAction action )
		{
			if ( OutputInvocationEvent != null )
			{
				OutputInvocationEvent( automaton, action );
			}
		}

		/// <summary>
		///   .
		/// </summary>
		private void LogSendMessage(
			Automaton targetAutomaton,
			Notification notification )
		{
			if ( SendMessageEvent != null )
			{
				SendMessageEvent( targetAutomaton, notification );
			}
		}

		/// <summary>
		///     .
		/// </summary>
		internal void LogEndNotificationProcessing(
			Automaton automaton,
			Notification notification )
		{
			if ( EndNotificationProcessingEvent != null )
			{
				EndNotificationProcessingEvent( automaton, notification );
			}
		}

		/// <summary>
		///    .
		/// </summary>
		private void LogEndCycle()
		{
			if ( EndCycleEvent != null )
			{
				EndCycleEvent( this );
			}
		}

		#endregion

		#region Log Events

		/// <summary>
		///    .
		/// </summary>
		public event CycleEventHandler BeginCycleEvent;

		/// <summary>
		///      .
		/// </summary>
		public event NotificationProcessingEventHandler
			BeginNotificationProcessingEvent;

		/// <summary>
		///    .
		/// </summary>
		public event InputInquireEventHandler InputInquireEvent;

		/// <summary>
		///    .
		/// </summary>
		public event NotificationCheckingEventHandler NotificationCheckingEvent;

		/// <summary>
		///       .
		/// </summary>
		public event StateCheckingEventHandler StateCheckingEvent;

		/// <summary>
		///   .
		/// </summary>
		public event TransitionEventHandler TransitionEvent;

		/// <summary>
		///    .
		/// </summary>
		public event OutputInvocationEventHandler OutputInvocationEvent;

		/// <summary>
		///   .
		/// </summary>
		public event SendMessageEventHandler SendMessageEvent;

		/// <summary>
		///     .
		/// </summary>
		public event NotificationProcessingEventHandler
			EndNotificationProcessingEvent;

		/// <summary>
		///    .
		/// </summary>
		public event CycleEventHandler EndCycleEvent;

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		///     .
		/// </summary>
		/// <param name="args">  .</param>
		public void CheckValid( params object[] args )
		{
			if ( m_IsValid )
			{
				return;
			}
			foreach ( Automaton a in Automatons )
			{
				a.CheckValid( m_Checkers );
			}
			m_IsValid = true;
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,     .
		/// </summary>
		public event ChangedEventHandler Changed;

		/// <summary>
		///   <see cref="Changed"/>.
		/// </summary>
		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			m_IsValid = false;
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}
