// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Reflection;
using System.Runtime.Serialization;

using LoCoL.Exceptions;

namespace LoCoL.Conditions
{
	/// <summary>
	///  .
	/// </summary>
	[Serializable]
	public class CheckNotification : Condition, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public CheckNotification( int notificationID )
			: this( notificationID, NotificationType.NotSpecified )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		public CheckNotification(
			int notificationID,
			NotificationType notificationType )
			: base( notificationID, notificationType )
		{
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected CheckNotification(
			SerializationInfo info,
			StreamingContext context )
			: base( info, context, typeof(int), typeof(NotificationType) )
		{
		}

		/// <summary>
		///    .
		/// </summary>
		public override void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			base.GetObjectData( info, context );
		}

		#endregion

		#region Overrided Condition Methods

		/// <summary>
		///  ,    .
		/// </summary>
		public override bool Evaluate( Automaton automaton )
		{
			bool result = automaton.Manager.CheckNotification(
				(int)this[ 0 ],
				(NotificationType)this[ 1 ] );
			automaton.Manager.LogNotificationChecking(
				automaton,
				automaton.Manager.Notifications[ (int)this[ 0 ] ].
					CreateNotification( (NotificationType)this[ 1 ] ),
				result );
			return result;
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		/// .
		/// </summary>
		/// <exception cref="CheckValidException">
		/// <list>
		/// <item><description>
		///      <see cref="Automaton"/>.
		/// </description></item>
		/// <item><description>
		///  ,   
		///      ID.
		/// </description></item>
		/// </list>
		/// </exception>
		public override void CheckValid( params object[] args )
		{
			if ( args.Length != 1 || !( args[ 0 ] is Automaton ) )
			{
				throw new CheckValidException( " " + 
					this.GetType().Name + "." +
					MethodInfo.GetCurrentMethod().Name + 
					"     Automaton." );
			}
			Automaton a = (Automaton)args[ 0 ];
			if ( a.Manager.Notifications[ (int)this[ 0 ] ] == null )
			{
				throw new CheckValidException( " , " + 
					"   " + a.ToString() +
					"     ID==" + this[ 0 ] + "." );
			}
		}

		#endregion

	}
}
