// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Runtime.Serialization;

namespace LoCoL.Collections
{
	/// <summary>
	///   .
	/// </summary>
	[Serializable]
	public class InputVariableSet : IEnumerable, IChangeable, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public InputVariableSet()
		{
			m_Set = new NamedObjectSet();
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///  .
		/// </summary>
		public InputVariableSet( InputVariableSet variables )
		{
			m_Set = variables.m_Set.Clone();
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected InputVariableSet(
			SerializationInfo info,
			StreamingContext context )
		{
			m_Set = (NamedObjectSet)info.GetValue(
				"Value",
				typeof(NamedObjectSet) );
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			info.AddValue( "Value", m_Set );
		}

		#endregion

		#region Private Members

		/// <summary>
		///    .
		/// </summary>
		private NamedObjectSet m_Set;

		#endregion

		#region Public Methods

		/// <summary>
		///  .
		/// </summary>
		public InputVariableSet Clone()
		{
			return new InputVariableSet( this );
		}

		/// <summary>
		///  .
		/// </summary>
		public void Clear()
		{
			m_Set.Clear();
		}

		/// <summary>
		///   .
		/// </summary>
		public void Add( InputVariable variable )
		{
			m_Set.Add( variable );
		}

		/// <summary>
		/// ,      .
		/// </summary>
		public bool Contains( InputVariable variable )
		{
			return m_Set.Contains( variable );
		}

		/// <summary>
		///     .
		///    ,  .
		/// </summary>
		public void Remove( InputVariable variable )
		{
			m_Set.Remove( variable );
		}

		#endregion

		#region Public Properties

		/// <summary>
		///      ID.
		/// ID -    ,      .
		/// </summary>
		public InputVariable this[ int id ]
		{
			get
			{
				return (InputVariable)m_Set[ id ];
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public int Count
		{
			get
			{
				return m_Set.Count;
			}
		}

		#endregion

		#region IEnumerable Implementation

		/// <summary>
		/// Enumerator.
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return m_Set.GetEnumerator();
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,     .
		/// </summary>
		public event ChangedEventHandler Changed;

		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}
