// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Runtime.Serialization;

namespace LoCoL.Collections
{
	/// <summary>
	///  .
	/// </summary>
	[Serializable]
	public class NotificationSet : IEnumerable, IChangeable, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public NotificationSet()
		{
			m_Set = new NamedObjectSet();
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///  .
		/// </summary>
		public NotificationSet( NotificationSet notifications )
		{
			m_Set = notifications.m_Set.Clone();
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected NotificationSet(
			SerializationInfo info,
			StreamingContext context )
		{
			m_Set = (NamedObjectSet)info.GetValue(
				"Value",
				typeof(NamedObjectSet) );
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			info.AddValue( "Value", m_Set );
		}

		#endregion

		#region Private Members

		/// <summary>
		///    .
		/// </summary>
		private NamedObjectSet m_Set;

		#endregion

		#region Public Methods

		/// <summary>
		///  .
		/// </summary>
		public NotificationSet Clone()
		{
			return new NotificationSet( this );
		}

		/// <summary>
		///  .
		/// </summary>
		public void Clear()
		{
			m_Set.Clear();
		}

		/// <summary>
		///  .
		/// </summary>
		public void Add( Notification notification )
		{
			m_Set.Add( notification );
		}

		/// <summary>
		/// ,     .
		/// </summary>
		public bool Contains( Notification notification )
		{
			return m_Set.Contains( notification );
		}

		/// <summary>
		///    .
		///   ,  .
		/// </summary>
		public void Remove( Notification notification )
		{
			m_Set.Remove( notification );
		}

		#endregion

		#region Public Properties

		/// <summary>
		///     ID.
		/// ID -   ,      .
		/// </summary>
		public Notification this[ int id ]
		{
			get
			{
				return (Notification)m_Set[ id ];
			}
		}

		/// <summary>
		///    .
		/// </summary>
		public int Count
		{
			get
			{
				return m_Set.Count;
			}
		}

		#endregion

		#region IEnumerable Implementation

		/// <summary>
		/// Enumerator.
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return m_Set.GetEnumerator();
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,     .
		/// </summary>
		public event ChangedEventHandler Changed;

		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion
	}
}
