// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Reflection;
using System.Runtime.Serialization;

using LoCoL.Collections;
using LoCoL.Exceptions;

namespace LoCoL
{
	/// <summary>
	///  .
	/// </summary>
	[Serializable]
	public class State :
		NamedObject, IValidCheckable, IChangeable, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		/// <param name="id">ID .</param>
		/// <param name="name"> .</param>
		public State( int id, string name )
			: this( id, name, null )
		{
		}

		/// <summary>
		/// .
		/// </summary>
		/// <param name="id">ID .</param>
		/// <param name="name"> .</param>
		/// <param name="actions">
		/// ,     .
		/// </param>
		public State( int id, string name, AutomatonActionArray actions )
			: base( id, name )
		{
			m_Actions =
				( actions == null ) ?
				( new AutomatonActionArray() ) :
				( actions.Clone() );
			m_Actions.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected State( SerializationInfo info, StreamingContext context )
			: base( info, context )
		{
			m_Actions = (AutomatonActionArray)info.GetValue(
				"Actions",
				typeof(AutomatonActionArray) );
		}

		/// <summary>
		///    .
		/// </summary>
		public override void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			base.GetObjectData( info, context );
			info.AddValue( "Actions", m_Actions );
		}

		#endregion


		#region Private Members

		/// <summary>
		/// ,     .
		/// </summary>
		private AutomatonActionArray m_Actions;

		#endregion

		#region Public Properties

		/// <summary>
		/// ,     .
		/// </summary>
		public AutomatonActionArray Actions
		{
			get
			{
				return m_Actions;
			}
		}

		#endregion

		#region IValidCheckable Implementation

		/// <summary>
		///   .
		/// </summary>
		/// <param name="args"> Automaton.</param>
		/// <exception cref="CheckValidException">
		///      <see cref="Automaton"/>.
		/// </exception>
		public void CheckValid( params object[] args )
		{
			if ( args.Length != 1 || !( args[ 0 ] is Automaton ) )
			{
				throw new CheckValidException( " " + 
					this.GetType().Name + "." +
					MethodInfo.GetCurrentMethod().Name + 
					"     Automaton." );
			}
			Automaton a = (Automaton)args[ 0 ];
			foreach ( IAutomatonAction aa in Actions )
			{
				aa.CheckValid( a );
			}
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,      .
		/// </summary>
		public event ChangedEventHandler Changed;

		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}
