// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;

using LoCoL.Collections;
using LoCoL.Exceptions;

namespace LoCoL
{
	/// <summary>
	/// ,     .
	/// </summary>
	public class StateAccessibilityAutomatonChecker : IAutomatonChecker
	{

		#region IAutomatonChecker Implementation

		/// <summary>
		///  .
		/// </summary>
		/// <remarks>
		///    
		///     (thread-safe).
		/// </remarks>
		/// <exception cref="CheckValidException">
		///     .
		/// </exception>
		public void Check( Automaton automaton )
		{
			lock ( this )
			{
				StateSet states = new StateSet( automaton.States );
				m_Transitions = CreateTransitions( automaton );
				Walk( automaton, states, automaton.States[ 0 ] );
				if ( states.Count > 0 )
				{
					string s = null;
					foreach ( State st in states )
					{
						s += ", " + st.ToString();
					}
					throw new CheckValidException( "  " +
						automaton.ToString() + "   " +
						": " + s.Substring( 2 ) + "." );
				}
			}
		}

		#endregion

		#region Private Members

		/// <summary>
		///     .
		/// </summary>
		private Hashtable m_Transitions;

		#endregion

		#region Private Methods

		/// <summary>
		///   <c>hash</c>,    <c>st</c>
		///   <c>t</c>.
		/// </summary>
		private void AddTransition( Hashtable hash, State st, Transition t )
		{
			if ( hash[ st ] == null )
			{
				hash[ st ] = new ArrayList();
			}
			ArrayList arr = (ArrayList)hash[ st ];
			arr.Add( t );
		}

		/// <summary>
		///      <c>automaton</c>.
		/// </summary>
		/// <returns>
		/// Hashtable      .
		/// </returns>
		private Hashtable CreateTransitions( Automaton automaton )
		{
			Hashtable hash = new Hashtable();
			foreach ( Transition t in automaton.Transitions )
			{
				if ( t.IsFromGroup )
				{
					foreach ( State st in automaton.Groups[ t.From ].States )
					{
						AddTransition( hash, st, t );
					}
				}
				else
				{
					State st = automaton.States[ t.From ];
					AddTransition( hash, st, t );
				}
			}
			return hash;
		}

		/// <summary>
		///        
		///  .
		/// </summary>
		/// <param name="automaton"> .</param>
		/// <param name="states"> .</param>
		/// <param name="state">
		/// ,    .
		/// </param>
		private void Walk( Automaton automaton, StateSet states, State state )
		{
			if ( !states.Contains( state ) )
			{
				return;
			}
			states.Remove( state );
			foreach ( Transition t in (IEnumerable)m_Transitions[ state ] )
			{
				Walk( automaton, states, automaton.States[ t.To ] );
			}
		}

		#endregion

	}
}
