// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Collections;
using System.Runtime.Serialization;

namespace LoCoL.Collections
{
	/// <summary>
	///  .
	/// </summary>
	[Serializable]
	public class TransitionSet : IEnumerable, IChangeable, ISerializable
	{

		#region Public Constructors

		/// <summary>
		/// .
		/// </summary>
		public TransitionSet()
		{
			m_Set = new NamedObjectSet();
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///  .
		/// </summary>
		public TransitionSet( TransitionSet transitions )
		{
			m_Set = transitions.m_Set.Clone();
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		#endregion

		#region ISerializable Implementation

		/// <summary>
		///  .
		/// </summary>
		protected TransitionSet(
			SerializationInfo info,
			StreamingContext context )
		{
			m_Set = (NamedObjectSet)info.GetValue(
				"Value",
				typeof(NamedObjectSet) );
			m_Set.Changed += new ChangedEventHandler( RaiseChangedEvent );
		}

		/// <summary>
		///    .
		/// </summary>
		public virtual void GetObjectData(
			SerializationInfo info,
			StreamingContext context )
		{
			info.AddValue( "Value", m_Set );
		}

		#endregion

		#region Private Members

		/// <summary>
		///    .
		/// </summary>
		private NamedObjectSet m_Set;

		#endregion

		#region Public Methods

		/// <summary>
		///  .
		/// </summary>
		public TransitionSet Clone()
		{
			return new TransitionSet( this );
		}

		/// <summary>
		///     ID.
		/// ID -   ,      .
		/// </summary>
		public Transition this[ int id ]
		{
			get
			{
				return (Transition)m_Set[ id ];
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public void Clear()
		{
			m_Set.Clear();
		}

		/// <summary>
		///  .
		/// </summary>
		public void Add( Transition transition )
		{
			m_Set.Add( transition );
		}

		/// <summary>
		/// ,     .
		/// </summary>
		public bool Contains( Transition transition )
		{
			return m_Set.Contains( transition );
		}

		/// <summary>
		///    .
		///   ,  .
		/// </summary>
		public void Remove( Transition transition )
		{
			m_Set.Remove( transition );
		}

		#endregion

		#region Public Properties

		/// <summary>
		///    .
		/// </summary>
		public int Count
		{
			get
			{
				return m_Set.Count;
			}
		}

		#endregion

		#region IEnumerable Implementation

		/// <summary>
		/// Enumerator.
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return m_Set.GetEnumerator();
		}

		#endregion

		#region IChangeable Implementation

		/// <summary>
		/// , ,     .
		/// </summary>
		public event ChangedEventHandler Changed;

		/// <summary>
		///   <see cref="Changed"/>.
		/// </summary>
		private void RaiseChangedEvent( object sender, ChangedEventArgs e )
		{
			if ( Changed != null )
			{
				Changed( this, e );
			}
		}

		#endregion

	}
}
