// Copyright:	(C) Alexander S. Naumov aka GooRoo, 2004.
// E-mail:		gooroo@bk.ru
// Last update:	23/05/2004

using System;
using System.Xml;

using LoCoL.Exceptions;

namespace LoCoL.Logs
{

	/// <summary>
	///      XML-.
	/// </summary>
	public class XmlLogger : ILogger, IDisposable
	{

		#region Public Constructor

		/// <summary>
		/// .
		///      .
		/// </summary>
		/// <param name="writer">
		/// <see cref="XmlWriter"/>,    .
		/// </param>
		/// <param name="manager">
		/// <see cref="AutomatonManager"/>,   
		/// <see cref="XmlLogger"/>.
		/// </param>
		/// <exception cref="LoggerException">
		/// <list>
		/// <item><description>
		/// <c>writer</c>    <c>null</c>.
		/// </description></item>
		/// <item><description>
		/// <c>manager</c>    <c>null</c>.
		/// </description></item>
		/// </list>
		/// </exception>
		public XmlLogger( XmlWriter writer, AutomatonManager manager )
		{
			if ( writer == null )
			{
				throw new LoggerException( "XmlWriter    null." );
			}
			if ( manager == null )
			{
				throw new LoggerException(
					"AutomatonManager    null." );
			}
			m_Writer = writer;
			m_Writer.WriteStartDocument();
			m_Manager = manager;
			m_Manager.RegisterLogger( this );
			m_Writer.WriteStartElement( "AutomatonLog" );
			m_Writer.WriteAttributeString(
				"Date",
				DateTime.Now.ToShortDateString() );
			if ( m_Manager.Title != null )
			{
				m_Writer.WriteAttributeString( "Title", m_Manager.Title );
			}
			if ( m_Manager.Version != null )
			{
				m_Writer.WriteAttributeString(
					"Version",
					m_Manager.Version.ToString() );
			}
			if ( m_Manager.Description != null )
			{
				m_Writer.WriteAttributeString(
					"Description",
					m_Manager.Description );
			}
			if ( m_Manager.Copyright != null )
			{
				m_Writer.WriteAttributeString(
					"Copyright",
					m_Manager.Copyright );
			}
		}

		#endregion

		#region Destructor

		/// <summary>
		/// .
		/// </summary>
		~XmlLogger()
		{
			Dispose();
		}

		#endregion

		#region Private Members

		/// <summary>
		/// <see cref="XmlWriter"/>,    .
		/// </summary>
		private XmlWriter m_Writer;

		/// <summary>
		/// ,   .
		/// </summary>
		private ILogDetailer m_LogDetailer;

		/// <summary>
		///  ,    <see cref="XmlLogger"/>.
		/// </summary>
		private AutomatonManager m_Manager;

		#endregion

		#region Public Properties

		/// <summary>
		/// <see cref="XmlWriter"/>,    .
		/// </summary>
		public XmlWriter Writer
		{
			get
			{
				return m_Writer;
			}
		}

		/// <summary>
		///  ,    <see cref="XmlLogger"/>.
		/// </summary>
		public AutomatonManager Manager
		{
			get
			{
				return m_Manager;
			}
		}

		#endregion

		#region ILogger Implementation

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnBeginCycle( AutomatonManager manager )
		{
			m_Writer.WriteStartElement( "Cycle" );
			m_Writer.WriteAttributeString(
				"Tick",
				manager.CurrentCycle.ToString() );
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledBeginCycle( manager ) )
			{
				return;
			}
		}

		/// <summary>
		///      .
		/// </summary>
		void ILogger.OnBeginNotificationProcessing(
			Automaton automaton,
			Notification notification )
		{
			m_Writer.WriteStartElement( "Automaton" );
			m_Writer.WriteAttributeString( "Name", automaton.ToString() );
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledBeginNotificationProcessing(
					automaton,
					notification ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "BeginNotificationProcessing" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString(
				"Notification",
				notification.ToString() );
			m_Writer.WriteAttributeString(
				"State",
				automaton.CurrentState.ToString() );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnInputInquire(
			Automaton automaton,
			InputVariable variable,
			bool result )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledInputInquire( automaton, variable ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "InputInquire" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString( 
				"InputVariable",
				variable.ToString() );
			m_Writer.WriteAttributeString(
				"Result",
				result ? "" : "" );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnNotificationChecking(
			Automaton automaton, 
			Notification notification,
			bool result )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledNotificationChecking(
					automaton,
				notification ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "NotificationChecking" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString(
				"Notification",
				notification.ToString() );
			m_Writer.WriteAttributeString(
				"Result", result ? "" : "" );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///       .
		/// </summary>
		void ILogger.OnStateChecking(
			Automaton automaton,
			Automaton checkedAutomaton,
			State state,
			bool result )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledStateChecking(
					automaton,
					checkedAutomaton,
					state ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "StateChecking" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString(
				"CheckedAutomaton",
				checkedAutomaton.ToString() );
			m_Writer.WriteAttributeString(
				"State",
				state.ToString() );
			m_Writer.WriteAttributeString(
				"Result",
				result ? "" : "" );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///   .
		/// </summary>
		void ILogger.OnTransition( Automaton automaton, Transition transition )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledTransition( automaton, transition ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "Transition" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString(
				"Transition",
				transition.ToString( automaton ) );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///   .
		/// </summary>
		void ILogger.OnSendMessage( Automaton targetAutomaton, Notification notification )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledSendMessage(
					targetAutomaton,
					notification ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "SendMessage" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString(
				"TargetAutomaton",
				targetAutomaton.ToString() );
			m_Writer.WriteAttributeString(
				"Message",
				notification.ToString() );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnOutputInvocation(
			Automaton automaton,
			OutputAction action )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledOutputInvocation( automaton, action ) )
			{
				return;
			}
			m_Writer.WriteStartElement( "OutputInvocation" );
			m_Writer.WriteAttributeString(
				"Time",
				DateTime.Now.ToLongTimeString() );
			m_Writer.WriteAttributeString(
				"OutputAction",
				action.ToString() );
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///     .
		/// </summary>
		void ILogger.OnEndNotificationProcessing(
			Automaton automaton,
			Notification notification )
		{
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledEndNotificationProcessing(
					automaton,
					notification ) )
			{
			}
			else
			{
				m_Writer.WriteStartElement( "EndNotificationProcessing" );
				m_Writer.WriteAttributeString(
					"Time",
					DateTime.Now.ToLongTimeString() );
				m_Writer.WriteAttributeString(
					"Notification",
					notification.ToString() );
				m_Writer.WriteAttributeString(
					"State",
					automaton.CurrentState.ToString() );
				m_Writer.WriteEndElement();
			}
			//   Automaton.
			m_Writer.WriteEndElement();
		}

		/// <summary>
		///    .
		/// </summary>
		void ILogger.OnEndCycle( AutomatonManager manager )
		{
			//   Cycle.
			m_Writer.WriteEndElement();
			if ( m_LogDetailer != null &&
				!m_LogDetailer.IsEnabledEndCycle( manager ) )
			{
				return;
			}
		}

		/// <summary>
		/// ,   .
		/// </summary>
		public ILogDetailer LogDetailer
		{
			get
			{
				return m_LogDetailer;
			}
			set
			{
				m_LogDetailer = value;
			}
		}

		#endregion

		#region IDisposable Implementation

		/// <summary>
		///    <see cref="Dispose"/>.
		/// </summary>
		private bool m_Disposed = false;
		/// <summary>
		///   .     .
		///   <see cref="Dispose"/>  
		///    .
		/// </summary>
		public void Dispose()
		{
			if ( !m_Disposed )
			{
				//   AutomatonLog.
				m_Writer.WriteEndElement();
				m_Writer.WriteEndDocument();
				m_Writer.Close();
				m_Disposed = true;
			}
		}

		#endregion
	}
}
