<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

	<xsl:template match="/AutomatonLog">
		<html>
			<head>
				<title>Протокол работы системы автоматов</title>
				<link rel="stylesheet" type="text/css" href="Default.css" />
				<script language="JavaScript" src="Default.js" />
			</head>
			<body>
				<div class="head">
					<h1>Протокол работы системы автоматов</h1>
					Название системы: <b><xsl:value-of select="@Title" /></b><br />
					Версия системы: <b><xsl:value-of select="@Version" /></b><br />
					Описание системы: <i><xsl:value-of select="@Description" /></i><br />
					Авторское право: <b><xsl:value-of select="@Copyright" /></b><br />
					Дата создания протокола: <b><xsl:value-of select="@Date" /></b>
				</div>
				<xsl:for-each select="Cycle">
					<xsl:apply-templates select="." />
				</xsl:for-each>
			</body>
		</html>
	</xsl:template>
	
	<xsl:template match="Cycle">
		<script language="JavaScript">BeginDivClass(0);</script>
		<i>[Начало цикла номер <b><xsl:value-of select="@Tick" /></b>]</i>
		<xsl:for-each select="Automaton">
			<xsl:apply-templates select="." />
		</xsl:for-each>
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>
	
	<xsl:template match="Automaton">
		<script language="JavaScript">deepNested++;</script>
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">BeginDivClass(1);</script>
		<script language="JavaScript">Indent();</script>
		<i>[Работа автомата <b><xsl:value-of select="@Name" /></b>]</i>
		<xsl:for-each select="BeginNotificationProcessing|InputInquire|NotificationChecking|StateChecking|Transition|SendMessage|Automaton|OutputInvocation|EndNotificationProcessing">
			<xsl:apply-templates select="." />
		</xsl:for-each>
		<script language="JavaScript">EndDivClass(1);</script>
		<script language="JavaScript">EndDivClass(0);</script>
		<script language="JavaScript">deepNested--;</script>
	</xsl:template>
	
	<xsl:template match="BeginNotificationProcessing">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат приступил к обработке уведомления <b><xsl:value-of select="@Notification"/></b> в состоянии <b><xsl:value-of select="@State"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="InputInquire">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат опросил входную переменную <b><xsl:value-of select="@InputVariable"/></b> и получил результат: <b><xsl:value-of select="@Result"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="NotificationChecking">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат сравнил переданное уведомление с <b><xsl:value-of select="@Notification"/></b> и получил результат: <b><xsl:value-of select="@Result"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="StateChecking">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат сравнил состояние автомата <b><xsl:value-of select="@CheckedAutomaton"/></b> с <b><xsl:value-of select="@State"/></b> и получил результат: <b><xsl:value-of select="@Result"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="Transition">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат совершил переход <b><xsl:value-of select="@Transition"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>
			
	<xsl:template match="OutputInvocation">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат выполнил выходное воздействие <b><xsl:value-of select="@OutputAction"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="SendMessage">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомату <b><xsl:value-of select="@TargetAutomaton"/></b> было послано сообщение <b><xsl:value-of select="@Message"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="EndNotificationProcessing">
		<script language="JavaScript">BeginDivClass(0);</script>
		<script language="JavaScript">Indent();</script>
		<xsl:apply-templates select="@Time" />
		Автомат завершил обработку уведомления <b><xsl:value-of select="@Notification"/></b> в состоянии <b><xsl:value-of select="@State"/></b>.
		<script language="JavaScript">EndDivClass(0);</script>
	</xsl:template>

	<xsl:template match="@Time">
		[<xsl:value-of select="." />]
	</xsl:template>
</xsl:stylesheet>
